
#include <asm_s3c2440.h>
#include <config.h>
	
.text

.global	_start
	
_start:	b	reset
	ldr	pc, _undefined_instruction
	ldr	pc, _software_interrupt
	ldr	pc, _prefetch_abort
	ldr	pc, _data_abort
	ldr	pc, _not_used
	ldr	pc, _irq
	ldr	pc, _fiq	

_undefined_instruction:	.word undefined_instruction
_software_interrupt:	.word software_interrupt
_prefetch_abort:	.word prefetch_abort
_data_abort:		.word data_abort
_not_used:		.word not_used
_irq:			.word irq
_fiq:			.word fiq

	.balignl 16,0xdeadbeef

	
		
reset:

	mrs	r0, cpsr
	bic	r0, r0, #0x1f
	orr	r0, r0, #0xd3	@ SVC mode AND IRQ/FIQ disable
	msr	cpsr, r0

	ldr	r0, =WTCON
	mov	r1, #0
	str	r1, [r0]

	ldr	r0, =INTMSK
	ldr	r1, =0xffffffff
	str	r1, [r0]

	ldr	r0, =INTSUBMSK
	ldr	r1, =0x7fff
	str	r1, [r0]
	
	bl	clksetup
	bl	memsetup	

@#ifdef DEBUG
	bl	_led_init
	bl	_led_off

@	mov	r0, #LED1
@	bl	_led_on
@#endif
	
	/*
	 * NOR FLASH ִ kernel image SDRAM 
	 */		
kernel_load:	
	ldr	r0, =KERNEL_IMAGE_START_NOR
	ldr	r1, =KERNEL_IMAGE_START_RAM
	ldr	r2, =KERNEL_IMAGE_END_NOR
	
copy_loop:
	ldmia	r0!, {r3-r10}
	stmia	r1!, {r3-r10}
	cmp	r0, r2
	blt	copy_loop
	
	/*
	 * kernel image load....
	 */
	ldr	pc, =KERNEL_IMAGE_START_RAM


undefined_instruction:
	ldr	pc, =pISR_UNDEF
	
software_interrupt:
	ldr	pc, =pISR_SWI
	
prefetch_abort:
	ldr	pc, =pISR_PABORT
	
data_abort:
	ldr	pc, =pISR_DABORT
	
not_used:
	ldr	pc, =pISR_RESERV
	
irq:
	ldr	pc, =pISR_INTREQ

fiq:
	ldr	pc, =pISR_FINTREQ


