
#ifndef __BLUE_KERNEL_H__
#define __BLUE_KERNEL_H__

#include <types.h>

/*
 * printk.c
 */
#define va_start(ap, p)		(ap = (char *) (&(p) + 1))
#define va_arg(ap, type)	((type *) (ap += sizeof(type)))[-1]
#define va_end(ap)		((void)0)

typedef char *va_list;

/*
 * led.S
 */
#define LED1	0x0040
#define LED2	0x0080

/*
 * uart.c
 */
void uart_init();
char uart_getc();
char *uart_gets(char *buffer, int size);
void uart_putc(const char c);
void uart_puts(const char *s);

/*
 * led.S
 */
void _led_init();
void _led_off();
void _led_on(int led);
void _led_wait_time();

/*
 * printk.c
 */
void printk(const char *, ...);
void print_dtob(int);
void print_dtoo(int);
void print_dtox(int);
void print_dec(int);

/*
 * interrupt.c
 */
void enable_interrupts();
int disable_interrupts();
void interrupt_init();
void do_undefined();
void do_software_interrupt();
void do_prefetch_abort();
void do_data_abort();
void do_reserved();
void do_irq(int intoffset);
void do_fiq();

/*
 * isr.c
 */
void timer4_init();
void timer4_isr();
void timer4_start();

/*
 * schedule.c 
 */
void schedule();

/*
 * uptime.c 
 */
void uptime_init();
unsigned long uptime_increase_ms();
void delay(unsigned int ms);

/*
 * common_asm.S 
 */
void _run_higher_prior_order_task();
unsigned int _get_current_cpsr();
unsigned int _get_current_spsr();
unsigned int _get_current_sp();

/*
 * version.c
 */
void version();
void info_developer();

/*
 * util.c
 */

#define kernel_panic()	\
	do		\
	{		\
		printk("kernel panic, file: %s, line: %d\n", __FILE__, __LINE__);	\
	} while(0)									\

unsigned int get_pc(void);
unsigned int get_sp(void);
unsigned int get_cpsr(void);
unsigned int get_spsr(void);

#endif
