
#ifndef __TCB_H__
#define __TCB_H__

#include <types.h>

#define DEFAULT_TASK_STACK	4096
#define TASK_MAX		32

typedef enum _task_state
{
	NEW = 0,
	READY,
	RUNNING,
	BLOCKING,
	STOP
} task_state_t;

typedef struct _task_tcb
{

	/*
	 * include/tcb.h
	 *
	 * task context frame
	 * ------------------
	 * context_frame[16]: lr_usr
	 * context_frame[15]: sp_usr
	 * context_frame[14]: r12
	 * context_frame[13]: r11
	 * context_frame[12]: r10
	 * context_frame[11]: r9
	 * context_frame[10]: r8
	 * context_frame[ 9]: r7
	 * context_frame[ 8]: r6
	 * context_frame[ 7]: r5
	 * context_frame[ 6]: r4
	 * context_frame[ 5]: r3
	 * context_frame[ 4]: r2
	 * context_frame[ 3]: r1
	 * context_frame[ 2]: r0
	 * context_frame[ 1]: pc
	 * context_frame[ 0]: spsr	<- curr_task
	 */
	unsigned int  context_frame[17];

	unsigned int  task_id;
	task_state_t  task_stat;
	unsigned char task_name[50];


	/*
	 * need_sched: default - 0, run schedule - 1
	 */
	unsigned int need_scheded;
	unsigned int time_slice;
	unsigned int time_remain;
	
	unsigned int time_wakeup;
	
	/*
	 * 켱  ٸ̾ ȭ ʿ
	 *  task 켱 real_prior( = prio + ageing) Ѵ
	 */
	unsigned int prior;
    
	/*
	 * stack   ̵ϴ 
	 * stmfd/ldmfd ϴ full descending 
	 * stack_top  ֻּ
	 * stack_bottom  ּ
	 * current stack pointer  task_context_frame ֽ
	 */
	unsigned int	stack_size;
	unsigned int	*stack_top;
	unsigned int	*stack_bottom;
	
	struct _task_tcb	*next;
	struct _task_tcb	*prev;
	
/* 	void (*task_func)(void); */
	
} tcb_t;

typedef struct _task_list
{
	tcb_t *head;
	tcb_t *tail;
	unsigned int task_count;
} task_list_t;


void task_init();
int task_create(unsigned char name[50], void (*task_func) (void), 
		unsigned int *stack, unsigned int stack_size, unsigned int prior, unsigned int time_slice);
void task_list_init(task_list_t *task_list);
void task_list_add(task_list_t *task_list, tcb_t *tcb, boolean last_node);
tcb_t *task_list_remove(task_list_t *task_list, unsigned int task_id);
tcb_t *task_list_get_first_node(task_list_t *task_list);
void task_list_viewer(task_list_t *task_list);

#endif
