
#include <asm_s3c2440.h>
#include <config.h>	
		
.global _start

_start:
	b	handler_reset
	ldr	pc, _handler_undefined
	ldr	pc, _handler_swi
	ldr	pc, _handler_prefetchabort
	ldr	pc, _handler_dataabort
	ldr	pc, _handler_reserved
	ldr	pc, _handler_irq
	ldr	pc, _handler_fiq

_handler_undefined:	.word	handler_undefined
_handler_swi:		.word	handler_swi
_handler_prefetchabort:	.word	handler_prefetchabort
_handler_dataabort:	.word	handler_dataabort
_handler_reserved:	.word	handler_reserved
_handler_irq:		.word	handler_irq
_handler_fiq:		.word	handler_fiq

	
handler_reset:

#ifdef DEBUG	
	mov	r0, #LED1
	bl	_led_on
#endif
	
	ldr	r0, =STACK_SVC_START
	mov	sp, r0				

	msr	cpsr_c, #NOINT|IRQ32MOD
	ldr	r0, =STACK_IRQ_START
	mov	sp, r0
	
	msr	cpsr_c, #NOINT|ABT32MOD
	ldr	r0, =STACK_ABORT_START
	mov	sp, r0

	msr	cpsr_c, #NOINT|FIQ32MOD
	ldr	r0, =STACK_FIQ_START
	mov	sp, r0
	
	msr	cpsr_c, #NOINT|UDF32MOD
	ldr	r0, =STACK_UDEF_START
	mov	sp, r0

	msr	cpsr_c, #NOINT|SVC32MOD

	
	ldr	pc, =start_kernel


handler_undefined:
	.word do_undefined

handler_swi:

	/*
	 * software interrupt exception
	 *  - ARM system developer's guide SLOS ҽ
	 */
	stmfd	sp!, {r0-r12, lr}	@ r0-r12, lr save svc mode's stack
	mrs	r1, spsr
	stmfd	sp!, {r1}
	
	ldr	r10, [lr, #-4]		@ swi number calc
	bic	r0, r10, #0xff000000
	bl	do_software_interrupt

	ldmfd	sp!, {r1}
	msr	spsr_cxsf, r1
	ldmfd	sp!, {r0-r12, pc}^
	
handler_prefetchabort:
	.word do_prefetch_abort

handler_dataabort:
	.word do_data_abort

handler_reserved:
	.word do_reserved

handler_irq:

	/*
	 * include/tcb.h
	 *
	 * task context frame
	 * ------------------
	 * context_frame[16]: spsr	: Non nested interrupt ʼ׸ ƴ   backup
	 * context_frame[15]: pc	
	 * context_frame[14]: lr_usr
	 * context_frame[13]: sp_usr
	 * context_frame[12]: r12
	 * context_frame[11]: r11
	 * context_frame[10]: r10
	 * context_frame[ 9]: r9
	 * context_frame[ 8]: r8
	 * context_frame[ 7]: r7
	 * context_frame[ 6]: r6
	 * context_frame[ 5]: r5
	 * context_frame[ 4]: r4
	 * context_frame[ 3]: r3
	 * context_frame[ 2]: r2
	 * context_frame[ 1]: r1
	 * context_frame[ 0]: r0	<- curr_task
	 */

#if	1

#if	0
	ldr	sp, =curr_task			@ curr_task's tcb pointer
	ldr	sp, [sp]

	sub	lr, lr, #4			@ calc pc

	stmea	sp!, {r0-r12, r13, r14}^	@ save r0-r12, sp_usr, lr_usr
	mrs	r0, spsr
	stmea	sp!, {lr, r0}			@ save pc, spsr
	
	ldr	sp, =STACK_IRQ_START		@ IRQ stack init
	
	ldr	r9, =INTOFFSET			@ calc interrupt offset / ISR Handler
	ldr	r9, [r9]
	ldr	r8, =_ISR_STARTADDRESS
	add	r8, r8, r9, lsl #2
	ldr	r8, [r8]

	mov	r0, r8				@ run ISR Handler
	bl	do_irq

	ldr	sp, =curr_task			@ curr_task's tcb pointer
	ldr	sp, [sp]			@ curr_task ٸ  ο ϼ ƴҼ ִ
	
	ldmea	sp!, {r0-r12, r13, r14}^	@ load r0-r12, sp_usr, lr_usr
	ldmea	sp!, {lr}			@ return task
						@ spsr only save, not load. then sp_irq need init
	mov	pc, lr
#else

	ldr	sp, =curr_task			@ curr_task's tcb pointer
	ldr	sp, [sp]

	sub	lr, lr, #4			@ calc pc

	add	sp, sp, #8
	stmia	sp, {r0-r12, r13, r14}^		@ save r0-r12, sp_usr, lr_usr
	
	mrs	r0, spsr
	stmdb	sp!, {r0, lr}			@ save pc, spsr
	
	ldr	sp, =STACK_IRQ_START		@ IRQ stack init

	ldr	r9, =INTOFFSET			@ calc interrupt offset / ISR Handler
	ldr	r9, [r9]
	ldr	r8, =_ISR_STARTADDRESS
	add	r8, r8, r9, lsl #2
	ldr	r8, [r8]

	mov	r0, r8				@ run ISR Handler
	bl	do_irq

	ldr	sp, =curr_task			@ curr_task's tcb pointer
	ldr	sp, [sp]			@ curr_task ٸ  ο ϼ ƴҼ ִ

	add	sp, sp, #8

	ldmdb	sp, {r0, lr}			@ load spsr, pc
	msr	spsr, r0

	ldmia	sp, {r0-r12, r13, r14}^		@ load r0-r12, sp_usr, lr_usr
	movs	pc, lr				@ return user_task
	
#endif
#else

	ldr	sp, =STACK_IRQ_START		@ IRQ stack init

	sub	lr, lr, #4
	stmfd	sp!, {r0-r3, r12, lr}

	ldr	r9, =INTOFFSET			@ calc interropt offset / ISR Handler
	ldr	r9, [r9]
	ldr	r8, =_ISR_STARTADDRESS
	add	r8, r8, r9, lsl #2
	ldr	r8, [r8]

	mov	r0, r8				@ run ISR Handler
	bl	do_irq

	ldmfd	sp!, {r0-r3, r12, pc}^
	
#endif	
	
handler_fiq:	
	.word do_fiq
     
