
#include <asm_s3c2440.h>
#include <c_s3c2440.h>
#include <config.h>	
#include <blue_kernel.h>
#include <tcb.h>
#include <systemcall.h>

/* enable IRQ interrupts */
void enable_interrupts (void)
{

	unsigned long temp;
	__asm__ __volatile__("mrs %0, cpsr\n"
			     "bic %0, %0, #0x80\n"
			     "msr cpsr_c, %0"
			     : "=r" (temp)
			     :
			     : "memory");
}

/*
 * disable IRQ/FIQ interrupts
 * returns true if interrupts had been enabled before we disabled them
 */
int disable_interrupts (void)
{

	unsigned long old,temp;
	__asm__ __volatile__("mrs %0, cpsr\n"
			     "orr %1, %0, #0xc0\n"
			     "msr cpsr_c, %1"
			     : "=r" (old), "=r" (temp)
			     :
			     : "memory");
	return (old & 0x80) == 0;
}

void interrupt_init()
{

	rINTMOD = 0x0;		/* All interrupt request is IRQ mode, not FIQ */
	rINTMSK = BIT_ALLMSK;

	enable_interrupts();


	timer4_init();		/* blue-kernel's timer tick */
}

/*
 * exceptions
 */
void do_undefined()
{

	printk("undefined\n");
	while(1)
	  ;
}

void do_software_interrupt(unsigned int swi_number)
{

/* 	switch( swi_number ) */
/* 	{ */
/* 	case __NR_test: */
		
}

void do_prefetch_abort()
{

	printk("prefetch abort\n");
	while(1)
	  ;
}

void do_data_abort()
{

	printk("data abort\n");
	while(1) 
	  ;
}

void do_reserved()
{

	printk("reserved\n");
	while(1)
	  ;
}

void do_irq(int intoffset)
{

	void (*pISR)();

	disable_interrupts();

	pISR = (void(*)())intoffset;
	pISR();

	enable_interrupts();
}

void do_fiq()
{

	printk("fiq\n");
	while(1)
	  ;
}
