
/*
 * ISR ϱ  disable interrupt   ISR handler  ʿ  
 */

#include <c_s3c2440.h>
#include <config.h>	
#include <blue_kernel.h>
#include <tcb.h>

#define TIMER4_PRESCALER	15
#define TIMER4_MUX		 3
#define TIMER4_DIVIDER		16


extern tcb_t *curr_task;
extern task_list[TASK_MAX];

extern task_list_t task_ready_list;

void timer4_init()
{

	pISR_TIMER4 = (unsigned) timer4_isr;
	rSRCPND	= BIT_TIMER4;
	rINTPND	= BIT_TIMER4;
	rINTMSK	&= ~(BIT_TIMER4);

#if 1
	/*
	 * TICK is 1000HZ (1ms)
	 * e.g: PCLK = (399650000/6) = 66.6MHz
	 * 
	 * priod = (prescaler value + 1) * (divider value) * buffer count / PCLK
	 * 1ms   = (15 + 1) * 16 * 260.156 / (66.6 * 1000 * 1000)
	 * 
	 * rTCNTB4 = 260.156 = 1ms * (66.6 * 1000 * 1000) / 16 / (15 + 1)
	 */
	rTCFG0 &= ~(0xff << 8);
	rTCFG0 |=  (TIMER4_PRESCALER << 8);

	rTCFG1 &= ~(0xff << 16);
	rTCFG1 |=  (TIMER4_MUX << 16);
	
	rTCNTB4 =  0.001 * (PCLK) / (TIMER4_DIVIDER) / (TIMER4_PRESCALER + 1);
    
	rTCON  &= ~(0x7  << 20);
	rTCON  |=  (0x3  << 21);	/* Interval mode(auto reload), Update TCNT4 */
	rTCON  &= ~(0x1  << 21);
#else
	/*
	 * TICK is 100HZ (10ms)
	 * e.g: PCLK = (399650000/6) = 66.6MHz
	 * 
	 * priod = (prescaler value + 1) * (divider value) * buffer count / PCLK
	 * 10ms   = (15 + 1) * 16 * 2601.56 / (66.6 * 1000 * 1000)
	 * 
	 * rTCNTB4 = 2601.56 = 10ms * (66.6 * 1000 * 1000) / 16 / (15 + 1)
	 */
	rTCFG0 &= ~(0xff << 8);
	rTCFG0 |=  (TIMER4_PRESCALER << 8);

	rTCFG1 &= ~(0xff << 16);
	rTCFG1 |=  (TIMER4_MUX << 16);
	
	rTCNTB4 =  0.01 * (PCLK) / (TIMER4_DIVIDER) / (TIMER4_PRESCALER + 1);
    
	rTCON  &= ~(0x7  << 20);
	rTCON  |=  (0x3  << 21);	/* Interval mode(auto reload), Update TCNT4 */
	rTCON  &= ~(0x1  << 21);
#endif
	/*
	 *  timer4 start ʾҴ. 
	 * timer4 main_tick ̾, task ʱȭ  Ϸ Ŀ startѴ
	 */
}

void timer4_start()
{

	rTCON  |=  (0x1  << 20);
}

void timer4_isr()
{

	clear_pending(BIT_TIMER4);

	/*
	 * update uptime_ms (1ms), uptime_sec (1000ms)
	 */
	uptime_increase_ms();

	/*
	 * system timer list ִ ͵߿ డ  ٷ ش
	 */
	/* TODO */

	/*
	 * WAITING ¿ ִ task ߿ waiting time  ͵ ready° ȴ
	 */
	/* TODO */

	/*
	 * ٸ ϰ Ǵ 
	 *  1. time_remain 0 
	 *  2. current task 켱  ready_queue ִ°   
	 */
	curr_task->time_remain--;
	if( curr_task->time_remain == 0 )
	{
		curr_task->need_scheded = 1;
		curr_task->time_remain = curr_task->time_slice;
	}

	if( curr_task->prior < task_ready_list.head->prior )
		curr_task->need_scheded = 2;

	if( curr_task->need_scheded )
		schedule();
}
