
#include <tcb.h>
#include <types.h>
#include <arm9.h>
#include <string.h>
#include <blue_kernel.h>
#include <config.h>


tcb_t *curr_task;
tcb_t *next_task;
tcb_t task_list[TASK_MAX];

task_list_t task_ready_list;
task_list_t task_blocking_list;

void task_init()
{

	int i, j;

	curr_task	= NULL;
	next_task	= NULL;

	task_list_init(&task_ready_list);
	task_list_init(&task_blocking_list);    

	for( i=0; i<TASK_MAX; i++)
	{
		for(j=0; j<(TASK_CONTEXT_FRAME/4); j++)
		{
			task_list[i].context_frame[j] = 0;
		}
		task_list[i].context_frame[0]	= NOFIQ|USR32MOD;
		
		task_list[i].task_id		= i;
		task_list[i].task_stat		= NEW;
		task_list[i].task_name[0]	= NULL;
		
		task_list[i].need_scheded	= 0;
		task_list[i].time_slice		= 0;
		task_list[i].time_remain	= 0;
		
		task_list[i].time_wakeup	= 0;
		
		task_list[i].prior		= 0;
	    
		task_list[i].stack_size		= 0;
		task_list[i].stack_top		= NULL;
		task_list[i].stack_bottom	= NULL;
		task_list[i].next		= NULL;
		task_list[i].prev		= NULL;
	    
/* 		task_list[i].task_func		= (void *)NULL; */
	}
}

int task_create(unsigned char name[50], void (*task_func) (void), 
		unsigned int *stack, unsigned int stack_size, unsigned int prior, unsigned int time_slice)
{

	int i;
    
	for( i=0; i<TASK_MAX; i++)
	{
		if( task_list[i].task_stat == NEW )
			break;
	}
    
	if( i >= TASK_MAX )
	{
		printk("* CRITICAL ERROR: task list full\n");
		while(1)
			;
	}

	task_list[i].task_stat		= READY;
	strcpy(task_list[i].task_name, name);

	task_list[i].prior		= prior;
	task_list[i].time_slice		= time_slice;
	task_list[i].time_remain	= time_slice;
    
	task_list[i].stack_size		= stack_size;
	task_list[i].stack_bottom	= stack;
	task_list[i].stack_top		= (unsigned int)stack + stack_size;
	task_list[i].context_frame[15]	= (unsigned int)stack + stack_size;	// context_frame[13]: task current stack pointer
/* 	task_list[i].task_func		= task_func; */
	task_list[i].context_frame[1]	= (unsigned int)task_func;		// context_frame[15]: program counter

	task_list_add(&task_ready_list, &task_list[i], FALSE);
	if( curr_task == NULL || curr_task->prior < task_list[i].prior )
	{
		curr_task = &task_list[i];
	}

	return task_list[i].task_id;
}


/*
 * task control block ϴ list
 *  - list ready/blocking list Ѵ
 *  -  list 켱(prior)  task ȴ
 *    (, prior Ϳ  ) 
 */
void task_list_init(task_list_t *task_list)
{

	task_list->head = NULL;
	task_list->tail = NULL;

	task_list->task_count = 0;
}

/*
 *  task 켱(prior)  list addѴ
 *  ̶, last_node TRUE 쿡    忡 ְ Ǵµ
 *  ̰ task time_slice μҺ  ٸ 켱  task
 *  ǵ ϱ ̴
 */
void task_list_add(task_list_t *task_list, tcb_t *tcb, boolean last_node)
{
  
	tcb_t *task_node_curr_position;

	if( task_list->task_count == 0 ) /* 尡 1   */
	{
		task_list->head = tcb;
		task_list->tail = tcb;

		tcb->prev = NULL;
		tcb->next = NULL;
		
		task_list->task_count = 1;
	}
	else if( last_node == TRUE ) /*  (1 ̻ 尡 ) 켱(proior)
				      *    ԵǾ   */
	{
		task_node_curr_position = task_list->tail;
		task_node_curr_position->next = tcb;
		tcb->prev  = task_node_curr_position;
		tcb->next  = NULL;
		task_list->tail = tcb;

		task_list->task_count++;
	}
	else
	{
		task_node_curr_position = task_list->head;
		if( task_node_curr_position->prior <= tcb->prior ) /* ù 忡 ԵǾ   */
		{
			tcb->next = task_node_curr_position;
			tcb->prev = NULL;

			task_list->head = tcb;
			task_node_curr_position->prev = tcb;

			task_list->task_count++;
		}
		else 
		{
			while( task_node_curr_position != NULL && task_node_curr_position->prior > tcb->prior )
			{
				task_node_curr_position = task_node_curr_position->next;
			}
		
			if( task_node_curr_position != NULL ) /*  ȿ ԵǾ   */
			{
				tcb->prev = task_node_curr_position->prev;
				tcb->next = task_node_curr_position;

				task_node_curr_position->prev->next = tcb;
				task_node_curr_position->prev = tcb;
			}
			else	/*   忡 ԵǾ   */
			{
				tcb->prev = task_list->tail;
				tcb->next = NULL;

				task_list->tail->next = tcb;
				task_list->tail = tcb;
			}

			task_list->task_count++;
		}
	}
}

tcb_t *task_list_remove(task_list_t *task_list, unsigned int task_id)
{

	tcb_t *task_node_curr_position;

	if( task_list->task_count < 1 )
		return NULL;

	task_node_curr_position = task_list->head;
	while( task_node_curr_position != NULL && task_node_curr_position->task_id != task_id )
	{
		task_node_curr_position = task_node_curr_position->next;
	}

	if( task_node_curr_position != NULL )
	{
		if( task_node_curr_position->prev == NULL )
		{
			/* first node */
			task_list->head = task_node_curr_position->next;
			task_node_curr_position->next->prev = NULL;
		}
		else if( task_node_curr_position->next == NULL)
		{
			/* last node */
			task_list->tail = task_node_curr_position->prev;
			task_node_curr_position->prev->next = NULL;
		}
		else
		{
			task_node_curr_position->prev->next = task_node_curr_position->next;
			task_node_curr_position->next->prev = task_node_curr_position->prev;
		}

		task_node_curr_position->next = NULL;
		task_node_curr_position->prev = NULL;

		task_list->task_count--;
	}
	else
	{

		while(1)
			kernel_panic();
	}

	return task_node_curr_position;
}

tcb_t *task_list_get_first_node(task_list_t *task_list)
{

	tcb_t *task;
	unsigned int first_task_id;
	
	if( task_list->head == NULL )
		return NULL;

	first_task_id = task_list->head->task_id;
	task = task_list_remove(task_list, first_task_id);

	return task;
}

void task_list_viewer(task_list_t *task_list)
{

	unsigned int i;
	tcb_t *temp;

	printk("task_list_viewer\n");
	printk("task_count: %d\n", task_list->task_count);
	printk("task head: %x\n", task_list->head);
	printk("task tail: %x\n", task_list->tail);

	i = 0;
	temp = task_list->head;
	while(temp)
	{
		printk("task %d: %x\n", i++, temp);
		temp = temp->next;
	}
}
